#include <LiquidCrystal.h> // Inclusion de la librairie pour afficheur LCD
#include <Keypad.h>
#include <Duinoedu_Utility.h>

//////////////////////////////////////////////
//        RemoteXY include library          //
//////////////////////////////////////////////

// RemoteXY select connection mode and include library
#define REMOTEXY_MODE__HARDSERIAL

#include <RemoteXY.h>

// RemoteXY connection settings
#define REMOTEXY_SERIAL Serial1
#define REMOTEXY_SERIAL_SPEED 9600


// RemoteXY configurate
#pragma pack(push, 1)
uint8_t RemoteXY_CONF[] =
{ 255, 12, 0, 34, 0, 54, 0, 13, 13, 1,
  1, 1, 12, 55, 24, 10, 2, 31, 79, 75,
  0, 129, 0, 3, 29, 18, 6, 17, 67, 111,
  100, 101, 32, 58, 32, 0, 65, 112, 47, 56,
  9, 9, 7, 36, 24, 28, 23, 7, 2, 26,
  2, 11, 67, 5, 12, 43, 43, 5, 1, 13,
  31
};

// this structure defines all the variables and events of your control interface
struct {

  // input variables
  uint8_t button_OK; // =1 if button pressed, else =0
  char edit_CODE[11];  // string UTF8 end zero

  // output variables
  uint8_t led_ACCES_r; // =0..255 LED Red brightness
  uint8_t led_ACCES_g; // =0..255 LED Green brightness
  uint8_t led_ACCES_b; // =0..255 LED Blue brightness
  char text_ETAT[31];  // string UTF8 end zero

  // other variable
  uint8_t connect_flag;  // =1 if wire connected, else =0

} RemoteXY;
#pragma pack(pop)

/////////////////////////////////////////////
//           END RemoteXY include          //
/////////////////////////////////////////////



const byte ROWS = 4; //4 lignes
const byte COLS = 3; //3 colonnes

char keys[ROWS][COLS] = {
  {'1', '2', '3'},
  {'4', '5', '6'},
  {'7', '8', '9'},
  {'*', '0', '#'}
};
byte rowPins[ROWS] = {28, 30, 32, 34}; //broches connectées à L0 -> L4
byte colPins[COLS] = {22, 24, 26}; //broches connectées à C0 -> C3


const int RS = 8; //declaration constante de broche RS
const int E = 9; //declaration constante de broche EN
const int LCDD4 = 10; //declaration constante de broche D4
const int LCDD5 = 11; //declaration constante de broche D5
const int LCDD6 = 12; //declaration constante de broche D6
const int LCDD7 = 13; //declaration constante de broche D7

LiquidCrystal lcd(RS, E, LCDD4, LCDD5, LCDD6, LCDD7);// Création d'un objet LiquidCrystal = initialisation LCD en mode 4 bits
Keypad keypad = Keypad( makeKeymap(keys), rowPins, colPins, ROWS, COLS );

const int INT = 2;

const int BUZZER = 3;
const int pinGACHE = A3;
const int pinON = A4;
const int pinOFF = A5;

byte clavier;
String CODE = "";
String CODE_Clavier = "1234";
int poids = 1000;
int n = 0;
char key;
boolean bSOUND = 1;

void SOUND(double, double);

//Variables surveillance batterie
int N = 0;
float Vin = 0.0;
float Ubat = 0;


void setup()
{
  RemoteXY_Init ();

  //initialisation liaison série
  Serial.begin(9600);

  //initialisation des broches en entrée
  pinMode(D0, INPUT);
  pinMode(D1, INPUT);
  pinMode(D2, INPUT);
  pinMode(D3, INPUT);
  pinMode(INT, INPUT);
  pinMode(A0, INPUT);

  //initialisation des broches en sortie
  pinMode(BUZZER, OUTPUT);
  pinMode(pinGACHE, OUTPUT);
  pinMode(pinON, OUTPUT);
  pinMode(pinOFF, OUTPUT);

  digitalWrite(pinGACHE, LOW);
  digitalWrite(pinON, LOW);
  digitalWrite(pinOFF, HIGH);

  lcd.begin(16, 2); // Initialise le LCD avec 16 colonnes x 2 lignes
  delay(10); // pause rapide pour laisser temps initialisation

  lcd.setCursor(0, 0);
  lcd.print("CODE : ---- ?   ") ;
  lcd.setCursor(0, 1);
  lcd.print("-ACCES INTERDIT-") ;
  CODE = "";
  poids = 1000;
  n = 0;

  SOUND(87, 2);
  SOUND(50, 4);
  SOUND(87, 3);

  RemoteXY.led_ACCES_r = 255;
  RemoteXY.led_ACCES_g = 0;
  RemoteXY.led_ACCES_b = 0;
  String smessage = "ACCES VERROUILLÉ";
  smessage.toCharArray(RemoteXY.text_ETAT, 30);
  RemoteXY_Handler();

  //transmission message debut programme
  Serial.println("Debut du programme ...");

}

void loop()
{
  String CODE_BT;
  bool bCODE_BT = false;
  RemoteXY_Handler();

  ONLY_EVERY(5000, _ABVAR_1_Repeat)
  //Surveillance_Batterie(); //appel de la fonction Surveillance_Batterie() toutes les 5 sec
  END_ONLY_EVERY

  if (RemoteXY.button_OK == 1) {
    Serial.println("Bouton OK !");
    CODE_BT = String(RemoteXY.edit_CODE);
    Serial.println("CODE BT = " + CODE_BT);
    RemoteXY.button_OK = 0;
    compareCODE(CODE_BT);
  }

  key = keypad.getKey();
  if (key != NO_KEY) {

    //on génère un son
    SOUND(39, 2);

    switch (key)
    {
      case '*' : //ANNULATION
        Serial.println('*');
        annulationCODE();
        break;

      case '#' : //VALIDATION
        Serial.println('#');
        compareCODE(CODE);
        break;

      default:
        if (n < 4) {
          //CODE = CODE + (int(key)-0x30) * poids;
          //poids = poids / 10;
          CODE = CODE + String(key);
          Serial.println("CODE : " + String(CODE));
          lcd.setCursor(7 + n, 0);  //Positionnement 7eme colonne - ligne 1
          lcd.print('*') ;
          //lcd.print(key) ;
          n++;
        }
        break;
    }
  }
}




void compareCODE(String code)
{

  Serial.println("Comparaison CODES : " + String(code) + " <-> " + String(CODE_Clavier));
  if (String(CODE_Clavier) == String(code)) {
    SOUND(25, 6);
    Serial.println("CODE OK");
    lcd.setCursor(0, 0);
    lcd.print("                ") ;
    lcd.setCursor(0, 1);
    lcd.print("-ACCES AUTORISE-") ;
    digitalWrite(pinGACHE, HIGH);
    digitalWrite(pinON, HIGH);
    digitalWrite(pinOFF, LOW);

    //modification interface Bluetooth
    String smessage = "ACCES AUTORISÉ";
    smessage.toCharArray(RemoteXY.text_ETAT, 30);
    RemoteXY.led_ACCES_g = 255;
    RemoteXY.led_ACCES_r = 0;
    RemoteXY.led_ACCES_b = 0;
    RemoteXY_Handler ();

    //temporisation de 5sec
    delay(5000);

  } else {
    SOUND(27, 6);
    lcd.setCursor(0, 0);
    lcd.print("                ") ;
    lcd.setCursor(0, 1);
    lcd.print("CODE INCORRECT !");

    Serial.println("CODE INCORRECT");
    delay(2000);
  }
  annulationCODE();
}

void annulationCODE()
{
  lcd.setCursor(0, 0);
  lcd.print("CODE : ---- ?   ") ;
  lcd.setCursor(0, 1);
  lcd.print("-ACCES INTERDIT-") ;
  CODE = "";
  poids = 1000;
  n = 0;
  digitalWrite(pinGACHE, LOW);
  digitalWrite(pinON, LOW);
  digitalWrite(pinOFF, HIGH);

  //modification interface mobile
  String smessage = "ACCES VERROUILLÉ";
  smessage.toCharArray(RemoteXY.text_ETAT, 30);
  RemoteXY.led_ACCES_r = 255;
  RemoteXY.led_ACCES_g = 0;
  RemoteXY.led_ACCES_b = 0;
  RemoteXY_Handler ();

  SOUND(15, 6);
}



void Surveillance_Batterie() {
  N = analogRead(A0);
  Vin = 5.0 * N / 1023.0;
  Ubat = 2.5 * Vin;
  Serial.println("N = " + String(N));
  Serial.println("Vin = " + String(Vin) + " V");
  Serial.println("Ubat = " + String(Ubat, 1) + " V");

  if (Ubat < 10.5) {
    SOUND(15, 6);
    lcd.setCursor(0, 1);
    lcd.print("BATTERY LOW !   ");
  }
  else {
    lcd.setCursor(0, 1);
    lcd.print("-ACCES INTERDIT-") ;
  }
}
