#include <LiquidCrystal.h> // Ajout de la librairie pour afficheur LCD
#include <Keypad.h>


const byte ROWS = 4; //4 lignes
const byte COLS = 3; //3 colonnes

char keys[ROWS][COLS] = {
  {'1', '2', '3'},
  {'4', '5', '6'},
  {'7', '8', '9'},
  {'*', '0', '#'}
};
byte rowPins[ROWS] = {28, 30, 32, 34}; //broches connectées à L0 -> L4
byte colPins[COLS] = {22, 24, 26}; //broches connectées à C0 -> C3


const int RS = 8; //declaration constante de broche RS
const int EN = 9; //declaration constante de broche EN
const int LCDD4 = 10; //declaration constante de broche D4
const int LCDD5 = 11; //declaration constante de broche D5
const int LCDD6 = 12; //declaration constante de broche D6
const int LCDD7 = 13; //declaration constante de broche D7

LiquidCrystal lcd(RS, EN, LCDD4, LCDD5, LCDD6, LCDD7);// Création d'un objet LiquidCrystal = initialisation LCD en mode 4 bits
Keypad keypad = Keypad( makeKeymap(keys), rowPins, colPins, ROWS, COLS );

const int BUZZER = 3;
const int pinGACHE = A3;
const int pinON = A4;
const int pinOFF = A5;

byte clavier;
String CODE = "";
String CODE_Clavier = "1234";
int n = 0;
char key;
boolean bSOUND = 1;

//Surveillance batterie
int N = 0;
float Vin = 0;
float Ubat = 0;

void SOUND(double, double);

void setup()
{
  //initialisation liaison série
  Serial.begin(9600);

  //initialisation des broches en sortie
  pinMode(BUZZER, OUTPUT);
  pinMode(pinGACHE, OUTPUT);
  pinMode(pinON, OUTPUT);
  pinMode(pinOFF, OUTPUT);

  digitalWrite(pinGACHE, LOW);
  digitalWrite(pinON, LOW);
  digitalWrite(pinOFF, HIGH);

  lcd.begin(16, 2); // Initialise le LCD avec 16 colonnes x 2 lignes
  delay(10); // pause rapide pour laisser temps initialisation

  lcd.setCursor(0, 0);
  lcd.print("CODE : ---- ?   ") ;
  lcd.setCursor(0, 1);
  lcd.print("-ACCES INTERDIT-") ;
  CODE = "";
  n = 0;

  SOUND(87, 2);
  SOUND(50, 4);
  SOUND(87, 3);

  //transmission message debut programme
  Serial.println("Debut du programme ...");

}


void loop()
{
  key = keypad.getKey();
  if (key != NO_KEY) {

    //on génère un son
    SOUND(39, 2);

    switch (key)
    {
      case '*' : //ANNULATION
        Serial.println('*');
        annulationCODE();
        break;

      case '#' : //VALIDATION
        Serial.println('#');
        compareCODE(CODE);
        break;

      default:
        if (n < 4) {
          CODE = CODE + String(key);
          Serial.print("CODE : "); Serial.println(CODE);
          lcd.setCursor(7 + n, 0);  //Positionnement 7eme colonne - ligne 1
          lcd.print('*') ;
          lcd.setCursor(0, 1);
          lcd.print("-ACCES INTERDIT-") ;
          //lcd.print(key) ;
          n = n + 1;
        }
        break;
    }
  }

  ONLY_EVERY(......, _ABVAR_1_Repeat)
  //A compléter !

  END_ONLY_EVERY


}

//Surveillance batterie
void SurveillanceBatterie()
{
  //A compléter !

  .... N = 0;
  .... Vin = 0;
  .... Ubat = 0;


}


void compareCODE(String code)
{
  Serial.println("Comparaison CODES : " + String(code) + " <-> " + String(CODE_Clavier));
  if (String(CODE_Clavier) == String(code)) {
    SOUND(25, 6);
    Serial.println("CODE OK");
    lcd.setCursor(0, 0);
    lcd.print("                ") ;
    lcd.setCursor(0, 1);
    lcd.print("-ACCES AUTORISE-") ;

    digitalWrite(pinGACHE, HIGH);
    digitalWrite(pinON, HIGH);
    digitalWrite(pinOFF, LOW);
    delay(5000);

  } else {
    SOUND(27, 6);
    lcd.setCursor(0, 0);
    lcd.print("                ") ;
    lcd.setCursor(0, 1);
    lcd.print("CODE INCORRECT !");

    Serial.println("CODE INCORRECT");
    delay(2000);
  }
  annulationCODE();
}


void annulationCODE()
{
  lcd.setCursor(0, 0);
  lcd.print("CODE : ---- ?   ") ;
  lcd.setCursor(0, 1);
  lcd.print("-ACCES INTERDIT-") ;
  CODE = "";
  n = 0;
  digitalWrite(pinGACHE, LOW);
  digitalWrite(pinON, LOW);
  digitalWrite(pinOFF, HIGH);
  SOUND(15, 6);
}
